@@ -3,10 +3,11 @@ Implementation of HTML-based dialogs.
 """
 
 from pscript import this_is_js
-from pscript.stubs import window, console, Math, isFinite, Date, isNaN
+from pscript.stubs import window, document, console, Math, isFinite, Date, isNaN
 
 
 if this_is_js():
+    tools = window.tools
     dt = window.dt
     utils = window.utils
 
@@ -21,7 +22,7 @@ def to_str(x):
 def show_background_div(show, keep_transparent=False):
     # Create element?
     if not window.dialogbackdiv:
-        window.dialogbackdiv = window.document.createElement("div")
+        window.dialogbackdiv = document.createElement("div")
         window.dialogbackdiv.className = "dialog-cover"
         document.getElementById("canvas").parentNode.appendChild(window.dialogbackdiv)
         # Make it block events
@@ -80,9 +81,7 @@ def _browser_history_popstate():
     if h.state and h.state.tt_state:
         if h.state.tt_state == 1:
             if len(stack) > 0:
-                h.pushState(
-                    {"tt_state": 2}, window.document.title, window.location.pathname
-                )
+                h.pushState({"tt_state": 2}, document.title, window.location.pathname)
                 stack[-1].close()
             else:
                 h.back()
@@ -100,12 +99,10 @@ def _browser_history_init():
     h = window.history
     if h.state and h.state.tt_state:
         if h.state.tt_state == 1:
-            h.pushState(
-                {"tt_state": 2}, window.document.title, window.location.pathname
-            )
+            h.pushState({"tt_state": 2}, document.title, window.location.pathname)
     else:
-        h.replaceState({"tt_state": 1}, window.document.title, window.location.pathname)
-        h.pushState({"tt_state": 2}, window.document.title, window.location.pathname)
+        h.replaceState({"tt_state": 1}, document.title, window.location.pathname)
+        h.pushState({"tt_state": 2}, document.title, window.location.pathname)
 
     # Now its safe to listen to history changes
     window.addEventListener("popstate", _browser_history_popstate, 0)
@@ -114,38 +111,6 @@ def _browser_history_init():
 _browser_history_init()
 
 
-def copy_dom_node(node):
-    global document
-
-    # Select the node (https://stackoverflow.com/questions/400212)
-    sel = None
-    if document.createRange and window.getSelection:  # FF, Chrome, Edge, ...
-        range = document.createRange()
-        sel = window.getSelection()
-        sel.removeAllRanges()
-        try:
-            range.selectNodeContents(node)
-            sel.addRange(range)
-        except Exception:
-            range.selectNode(node)
-            sel.addRange(range)
-    elif document.body.createTextRange:  # IE?
-        range = document.body.createTextRange()
-        range.moveToElementText(node)
-        range.select()
-
-    # Make a copy
-    try:
-        successful = window.document.execCommand("copy")
-    except Exception:
-        successful = False
-
-    if not successful:
-        return  # Don't unselect, user can now copy
-    if sel is not None:
-        sel.removeAllRanges()
-
-
 def csvsplit(s, sep, i=0):
     """Split a string on the given sep, but take escaping with double-quotes
     into account. Double-quotes themselves can be escaped by duplicating them.
@@ -230,7 +195,7 @@ class BaseDialog:
         self._callback = None
 
     def _create_main_div(self):
-        self.maindiv = window.document.createElement("form")
+        self.maindiv = document.createElement("form")
         self.maindiv.addEventListener("keydown", self._on_key, 0)
         self._canvas.node.parentNode.appendChild(self.maindiv)
         self.maindiv.className = "dialog"
@@ -381,14 +346,10 @@ class MenuDialog(BaseDialog):
         # Put the menu right next to the menu button
         self.maindiv.style.top = "5px"
         self.maindiv.style.left = "50px"
+        self.maindiv.style.maxWidth = "500px"
 
         self.maindiv.innerHTML = f"""
             <div class='loggedinas'></div>
-            <div class='divider'></div>
-            <a href="/"><img style='width:20px; height:20px;vertical-align:middle;' src='timetagger_sd.svg' />&nbsp;&nbsp;Homepage</a>
-            <a href="https://timetagger.app/support"><i class='fas'>\uf059</i>&nbsp;&nbsp;Get tips and help</a>
-            <a href="../account"><i class='fas'>\uf2bd</i>&nbsp;&nbsp;Account</a>
-            <div class='divider'></div>
         """.rstrip()
 
         # Unpack
@@ -400,21 +361,9 @@ class MenuDialog(BaseDialog):
         else:
             store_valid = True
             logged_in = False
-            # logged_in = bool(window.auth.get_auth_info())
-        logged_in  # noqaM
 
         is_installable = window.pwa and window.pwa.deferred_prompt
 
-        # <a href="login"><i class='fas'>\uf2f6</i>&nbsp;&nbsp;Login</a>
-        # <a href="logout"><i class='fas'>\uf2f5</i>&nbsp;&nbsp;Logout</a>
-        # # Hide login or logout button, or both
-        # loginbut = self.maindiv.children[3]
-        # logoutbut = self.maindiv.children[4]
-        # if logged_in:
-        #     loginbut.style.display = "none"
-        # else:
-        #     logoutbut.style.display = "none"
-
         # Display sensible text in "header"
         if window.store.__name__.startswith("Demo"):
             text = "This is the Demo"
@@ -423,36 +372,54 @@ class MenuDialog(BaseDialog):
         elif window.store.get_auth:
             auth = window.store.get_auth()
             if auth:
-                text = "Signed in as " + auth.email
+                text = "Signed in as " + auth.username
             else:
                 text = "Not signed in"
-        if window.timetaggerversion:
-            text += " - TimeTagger " + window.timetaggerversion
         loggedinas.innerText = text
 
+        whatsnew = "What's new"
+        whatsnew_url = "https://github.com/almarklein/timetagger/releases"
+        if window.timetaggerversion:
+            whatsnew += " in version " + window.timetaggerversion.lstrip("v")
+
         container = self.maindiv
-        for icon, isvalid, title, func in [
-            ("\uf013", store_valid, "Settings", self._show_settings),
+        for icon, show, title, func in [
+            (None, True, "External pages", None),
+            ("\uf015", True, "Homepage", "/"),
+            ("\uf059", True, "Get tips and help", "https://timetagger.app/support"),
+            ("\uf0a1", True, whatsnew, whatsnew_url),
+            (None, store_valid, "Manage", None),
             ("\uf02c", store_valid, "Search & manage tags", self._manage_tags),
             ("\uf56f", store_valid, "Import records", self._import),
             ("\uf56e", store_valid, "Export all records", self._export),
-            (
-                "\uf3fa",
-                is_installable,
-                "<span class='acc_color'>Install this app</span>",
-                self._do_install,
-            ),
+            (None, True, "User", None),
+            ("\uf013", store_valid, "Settings", self._show_settings),
+            ("\uf2bd", True, "Account", "../account"),
+            ("\uf2f6", not logged_in, "Login", "../login"),
+            ("\uf2f5", logged_in, "Logout", "../logout"),
+            (None, is_installable, None, None),
+            ("\uf3fa", is_installable, "<b>Install this app</b>", self._do_install),
         ]:
-            if not isvalid:
+            if not show:
                 continue
-            elif title is None:
-                el = window.document.createElement("div")
+            elif not func:
+                # Divider
+                el = document.createElement("div")
                 el.setAttribute("class", "divider")
+                if title is not None:
+                    el.innerHTML = title
                 container.appendChild(el)
             else:
-                el = window.document.createElement("a")
-                el.innerHTML = f"<i class='fas'>{icon}</i>&nbsp;&nbsp;{title}"
-                el.onclick = func
+                el = document.createElement("a")
+                html = ""
+                if icon:
+                    html += f"<i class='fas'>{icon}</i>&nbsp;&nbsp;"
+                html += title
+                el.innerHTML = html
+                if isinstance(func, str):
+                    el.href = func
+                else:
+                    el.onclick = func
                 container.appendChild(el)
 
         # more: Settings, User account, inport / export
@@ -1076,7 +1043,7 @@ class RecordDialog(BaseDialog):
         # Enable for some more info (e.g. during dev)
         if False:
             for x in [f"ID: {record.key}", f"Modified: {dt.time2localstr(record.mt)}"]:
-                el = window.document.createElement("div")
+                el = document.createElement("div")
                 el.innerText = x
                 self.maindiv.appendChild(el)
 
@@ -1816,7 +1783,7 @@ class ReportDialog(BaseDialog):
         self._canvas.record_dialog.open("Edit", record, self._update_table)
 
     def _copy_clipboard(self):
-        copy_dom_node(self._table_element)
+        tools.copy_dom_node(self._table_element)
         self._copy_but.innerHTML = (
             f"<i class='fas'>\uf46c</i>&nbsp;&nbsp;{self._copybuttext}"
         )
@@ -1841,7 +1808,7 @@ class ReportDialog(BaseDialog):
         if window.store.get_auth:
             auth = window.store.get_auth()
             if auth:
-                user = auth.email  # noqa
+                user = auth.username  # noqa
 
         for row in rows:
             if row[0] == "blank":
@@ -2061,7 +2028,7 @@ class ExportDialog(BaseDialog):
     async def fill_records(self):
         self._working += 1
         working = self._working
-        await window.stores.sleepms(100)
+        await window.tools.sleepms(100)
 
         # Prepare
         self._copy_but.disabled = True
@@ -2095,7 +2062,7 @@ class ExportDialog(BaseDialog):
             if len(lines) % 256 == 0:
                 self._copy_but.innerHTML = "Found " + len(lines) + " records"
                 # self._table_element.innerHTML = lines.join("\n")
-                await window.stores.sleepms(1)
+                await window.tools.sleepms(1)
             if working != self._working:
                 return
 
@@ -2106,7 +2073,7 @@ class ExportDialog(BaseDialog):
 
     def _copy_clipboard(self):
         table = self.maindiv.children[-1]
-        copy_dom_node(table)
+        tools.copy_dom_node(table)
         self._copy_but.innerHTML = "Copy export-table <i class='fas'>\uf46c</i>"
         window.setTimeout(self._reset_copy_but_text, 800)
 
@@ -2425,7 +2392,7 @@ class ImportDialog(BaseDialog):
                 # Keep giving feedback / dont freeze
                 if row % 100 == 0:
                     self._import_but.innerHTML = f"Found {len(records)} records"
-                    await window.stores.sleepms(1)
+                    await window.tools.sleepms(1)
             except Exception as err:
                 log(f"Error at row {row}: {err}")
                 return
@@ -2521,7 +2488,7 @@ class SettingsDialog(BaseDialog):
         # Darkmode
         self._darkmode_select.onchange = self._on_darkmode_change
         ob = window.store.settings.get_by_key("darkmode")
-        self._darkmode_select.value = 0 if ob is None else ob.value
+        self._darkmode_select.value = 1 if ob is None else ob.value
 
         # Stopwatch
         self._stopwatch_check = self._stopwatch_label.children[0]
@@ -2544,36 +2511,3 @@ class SettingsDialog(BaseDialog):
         stopwatch = bool(self._stopwatch_check.checked)
         ob = window.store.settings.create("stopwatch", stopwatch)
         window.store.settings.put(ob)
-
-
-class InstallInstructionsDialog(BaseDialog):
-    """Dialog to show install instructions."""
-
-    def __init__(self, canvas):
-        super().__init__(canvas)
-
-    def open(self, callback=None):
-        html = f"""
-            <h1><i class='fas'>\uf3fa</i>&nbsp;&nbsp;How to install
-                <button type='button'><i class='fas'>\uf00d</i></button>
-            </h1>
-            <p>
-            The TimeTagger app is a Progressive Web App (PWA), which means that
-            it can be installed as a native app on most devices without using
-            the official appstore/playstore.
-            </p><p>
-            On mobile devices, the browser may prompt you with the option to
-            "Add to homescreen". Otherwise, look for this option in the browser's
-            menu.
-            </p><p>
-            On desktop, open the app in the Chrome browser and select
-            "Install TimeTagger App" from the menu.
-            </p>
-            """
-
-        self.maindiv.innerHTML = html
-
-        self._cancel_but = self.maindiv.children[0].children[-1]
-        self._cancel_but.onclick = self.close
-
-        super().open(callback)
