@@ -23,10 +23,11 @@ When do ranges overlap?
 """
 
 from pscript import this_is_js
-from pscript.stubs import Math, Date, JSON, window, location, console, RawJS
+from pscript.stubs import Math, Date, JSON, window, console, RawJS
 
 
 if this_is_js():  # pragma: no cover
+    tools = window.tools
     utils = window.utils
     dt = window.dt  # noqa
     random = Math.random
@@ -173,17 +174,6 @@ def make_hidden(item):
     item.ds = "HIDDEN " + item.get("ds", "").split("HIDDEN")[-1].strip()
 
 
-def sleepms(ms):
-    global RawJS
-    return RawJS("new Promise(resolve => setTimeout(resolve, ms))")
-
-
-def build_api_url(suffix):
-    url = location.protocol + "//" + location.hostname + ":" + location.port
-    url = url.rstrip(":") + location.pathname.split("/app/")[0] + "/api/v1/"
-    return url + suffix
-
-
 # %% Sub stores
 
 
@@ -744,23 +734,29 @@ class ConnectedDataStore(BaseDataStore):
         self._server_time = 0
         self._last_auth_get = 0
         self._pull_statuses = [0, 0, 0, 0, 0]
-        self._auth = window.auth.get_auth_info()
+        self._auth = window.tools.get_auth_info()
+        self._auth_cantuse = None
 
     def get_auth(self):
-        """Get an auth info object that is guaranteed to match the email
+        """Get an auth info object that is guaranteed to match the username
         that the store had from the beginning. It gets automatically refreshed
         to include the latest authtoken. Can return None, in which case the store is
         not usable.
         """
         if dt.now() - self._last_auth_get > 1.0:
             self._last_auth_get = dt.now()
-            auth = window.auth.get_auth_info()
+            auth = window.tools.get_auth_info()
             if self._auth is None and auth:
-                window.location.reload()  # Reload when we login after having been logged out
-            elif auth and auth.email and auth.email == self._auth.email:
+                pass  # This can't really happen. If it does, let user reload
+            elif auth and auth.username and auth.username == self._auth.username:
                 self._auth = auth
             else:
                 self._auth = None
+            # Check if we've found the auth is not usable (e.g. expiration)
+            if self._auth:
+                if self._auth_cantuse:
+                    self._auth.cantuse = self._auth_cantuse
+
         return self._auth
 
     def _log_load(self, where, ob):
@@ -769,14 +765,14 @@ class ConnectedDataStore(BaseDataStore):
             console.log("Loading from " + where + ": " + [n1, n2, n3])
 
     async def _clear_cache(self):
-        storage = window.utils.AsyncStorage()
+        storage = window.tools.AsyncStorage()
         await storage.clear()
 
     async def _load_from_cache(self):
-        if self._auth and self._auth.email:
+        if self._auth and self._auth.username:
             try:
-                storage = window.utils.AsyncStorage()
-                ob = await storage.getItem(self._auth.email)
+                storage = window.tools.AsyncStorage()
+                ob = await storage.getItem(self._auth.username)
                 if ob and ob.server_time:
                     self._log_load("cache", ob)
                     self._server_time = ob.server_time
@@ -792,15 +788,15 @@ class ConnectedDataStore(BaseDataStore):
                 console.warn(err)
 
     async def _save_to_cache(self):
-        if self._auth and self._auth.email:
+        if self._auth and self._auth.username:
             try:
                 dump = {
-                    "key": self._auth.email,
+                    "key": self._auth.username,
                     "server_time": self._server_time,
                     "settings": self.settings.get_dump(),
                     "records": self.records.get_dump(),
                 }
-                storage = window.utils.AsyncStorage()
+                storage = window.tools.AsyncStorage()
                 await storage.setItem(dump)
             except Exception as err:
                 console.warn(err)
@@ -808,7 +804,8 @@ class ConnectedDataStore(BaseDataStore):
     async def _sync(self):
         # Ignore sync if there is no auth info
         auth = self.get_auth()
-        if not auth:
+        if not auth or auth.cantuse:
+            self._set_state("error")
             return
         # Get from local cache?
         if self._server_time == 0:
@@ -822,7 +819,7 @@ class ConnectedDataStore(BaseDataStore):
 
     async def _force_reset(self):
         # Set the reset flag at the server. Intended for testing.
-        url = build_api_url("forcereset")
+        url = tools.build_api_url("forcereset")
         authtoken = self.get_auth().token
         init = dict(method="PUT", headers={"authtoken": authtoken})
         await window.fetch(url, init)
@@ -836,7 +833,7 @@ class ConnectedDataStore(BaseDataStore):
         self._to_push[kind] = {}
 
         # Fetch and wait for response
-        url = build_api_url(kind)
+        url = tools.build_api_url(kind)
         init = dict(
             method="PUT",
             body=JSON.stringify(items.values()),
@@ -867,7 +864,7 @@ class ConnectedDataStore(BaseDataStore):
             # will get the server's version back when we pull.
             d = JSON.parse(await res.text())
             # d.accepted -> list of ok keys
-            for key in d.fail:
+            for key in d.failed:
                 self[kind]._drop(key)
             for err in d.errors:
                 self._set_state("warning")
@@ -876,7 +873,7 @@ class ConnectedDataStore(BaseDataStore):
     async def _pull(self, authtoken):
 
         # Fetch and wait for response
-        url = build_api_url("updates?since=" + self._server_time)
+        url = tools.build_api_url("updates?since=" + self._server_time)
         init = dict(method="GET", headers={"authtoken": authtoken})
         try:
             res = await window.fetch(url, init)
@@ -887,13 +884,19 @@ class ConnectedDataStore(BaseDataStore):
 
         # Process response
         if res.status != 200:
-            console.warn(res.status + " (" + res.statusText + ") " + await res.text())
+            text = await res.text()
+            console.warn(res.status + " (" + res.statusText + ") " + text)
             self._set_state("error")  # E.g. Wifi or server down, or 500
-            if res.status == 403:
-                if self._pull_statuses[-2] != 403 and self._pull_statuses[-3] != 403:
-                    # We do have internet, but token is invalid: renew and sync sooner
-                    await window.auth.renew_maybe()
-                    self.sync_soon(4)
+            if res.status == 401:
+                # Our token is probably expired. There may be local
+                # changes that have not yet been pushed, which would
+                # be lost if we logout. On the other hand, this may be
+                # a lost/stolen device and the user revoked the token.
+                # We can distinguish between these cases by determining
+                # that the 401 is due to a token seed mismatch (revoked).
+                self._auth_cantuse = text
+                if "revoked" in text:
+                    window.location.href = "../logout"
         else:
             ob = JSON.parse(await res.text())
             if ob.server_time:
@@ -935,7 +938,7 @@ class SandboxDataStore(BaseDataStore):
             items = self._to_push[kind].values()
             self._to_push[kind] = {}
             if len(items) > 0:
-                await sleepms(200)
+                await tools.sleepms(200)
                 for item in items:
                     item.st = dt.now()
                 self[kind]._put_received(*items)
@@ -962,7 +965,7 @@ class DemoDataStore(BaseDataStore):
             items = self._to_push[kind].values()
             self._to_push[kind] = {}
             if len(items) > 0:
-                await sleepms(200)
+                await tools.sleepms(200)
                 for item in items:
                     item.st = dt.now()
                 self[kind]._put_received(*items)
@@ -971,7 +974,7 @@ class DemoDataStore(BaseDataStore):
         while len(self._years) > 0:
             self._create_one_year_of_data(self._years.pop(-1))
         else:
-            await sleepms(200)
+            await tools.sleepms(200)
 
     def _create_tags(self):
 
