@@ -750,94 +750,6 @@ class BaseCanvas:
         pass
 
 
-class AsyncStorage:
-    """A kind of localstorage API, but async and without the 5MB memory
-    restriction, based on IndexedDB.
-    """
-
-    _dbname = "timeturtle"
-    _dbstorename = "cache"
-    _dbversion = 1
-
-    async def clear(self):
-        """Async delete all items from the cache."""
-
-        def executor(resolve, reject):
-            on_error = lambda e: reject(self._error_msg(e))
-
-            def on_db_ready(e):
-                db = e.target.result
-                db.onerror = on_error
-                transaction = db.transaction([self._dbstorename], "readwrite")
-                request = transaction.objectStore(self._dbstorename).clear()
-                request.onsuccess = lambda: resolve(None)
-
-            request = window.indexedDB.open(self._dbname, self._dbversion)
-            request.onerror = on_error
-            request.onupgradeneeded = self._on_update_required
-            request.onsuccess = on_db_ready
-
-        return await window.Promise(executor)
-
-    async def setItem(self, ob):
-        """Async put an object in the db."""
-        if not ob.key:
-            raise KeyError("Object must have a 'key' property")
-
-        def executor(resolve, reject):
-            on_error = lambda e: reject(self._error_msg(e))
-
-            def on_db_ready(e):
-                db = e.target.result
-                db.onerror = on_error
-                transaction = db.transaction([self._dbstorename], "readwrite")
-                request = transaction.objectStore(self._dbstorename).put(ob)
-                request.onsuccess = lambda: resolve(None)
-
-            request = window.indexedDB.open(self._dbname, self._dbversion)
-            request.onerror = on_error
-            request.onupgradeneeded = self._on_update_required
-            request.onsuccess = on_db_ready
-
-        return await window.Promise(executor)
-
-    async def getItem(self, key):
-        """Async get an object from the db."""
-
-        def executor(resolve, reject):
-            on_error = lambda e: reject(self._error_msg(e))
-
-            def on_db_ready(e):
-                db = e.target.result
-                db.onerror = on_error
-                transaction = db.transaction([self._dbstorename], "readonly")
-                request = transaction.objectStore(self._dbstorename).get(key)
-                request.onsuccess = lambda e: resolve(e.target.result)
-
-            request = window.indexedDB.open(self._dbname, self._dbversion)
-            request.onerror = on_error
-            request.onupgradeneeded = self._on_update_required
-            request.onsuccess = on_db_ready
-
-        return await window.Promise(executor)
-
-    def _error_msg(self, e):
-        msg = "IndexDB error"
-        if e.target.errorCode:
-            msg += " (" + e.target.errorCode + ")"
-        if e.target.error:
-            msg += ": " + e.target.error
-        return msg
-
-    def _on_update_required(self, e):
-        # This is where we structure the database.
-        # Gets called before db_open_request.onsuccess.
-        db = e.target.result
-        for i in range(len(db.objectStoreNames)):
-            db.deleteObjectStore(db.objectStoreNames[i])
-        db.createObjectStore(self._dbstorename, {"keyPath": "key"})
-
-
 if __name__ == "__main__":
     import pscript
 
